package org.expeditee.io;

import java.io.File;
import java.io.IOException;

import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.items.Item;

public abstract class AbstractHTMLWriter extends DefaultTreeWriter {
	private int _indent = 0;

	private static final String INDENT = "   ";

	private static final String FILE_FOLDER_SUFFIX = "_files";

	@Override
	protected void initialise(Frame start) throws IOException {
		_format = ".html";
		super.initialise(start);

		_writer.write("<html>" + ItemWriter.NEW_LINE);
		_writer.write("<head>" + ItemWriter.NEW_LINE);
		_writer.write("<title>" + start.getTitle());
		_writer.write("</title>" + ItemWriter.NEW_LINE);
		_writer.write("</head>" + ItemWriter.NEW_LINE);
		_writer.write("<body>" + ItemWriter.NEW_LINE);
	}

	private String _filesFolderName = null;

	protected String getFilesFolder() {
		if (_filesFolderName == null) {
			_filesFolderName = _filename.substring(0, _filename.lastIndexOf('.')) + FILE_FOLDER_SUFFIX;
			File dir = new File (_filesFolderName);
			_filesFolderName = dir.getName();
			if (!dir.exists()) {
				try {
					dir.mkdirs();
				} catch (Exception e) {
					FrameGraphics.ErrorMessage("Could not create folder: "
							+ dir.getName());
					_filesFolderName = null;
					return "";
				}
			}
		}
		
		return _filesFolderName + File.separator;
	}

	@Override
	protected String finaliseTree() throws IOException {
		_writer.write("</body>" + ItemWriter.NEW_LINE);
		return super.finaliseTree();
	}

	@Override
	protected void writeStartLink(Item linker) throws IOException {
		_indent++;
	}

	@Override
	protected void writeEndLink(Item linker) throws IOException {
		_indent--;
	}

	protected void indent() throws IOException {
		for (int i = 0; i < _indent; i++)
			_writer.write(INDENT);
	}

	protected int getIndent() {
		return _indent;
	}
}
